#!/usr/bin/perl -w
# $Id: Util.pm,v 1.1 2000/06/28 10:03:09 thockin Exp $
# author: thockin@cobalt.com

package AM::Util;

use strict;
use IO::Socket;
require Exporter;

use vars qw(@ISA @EXPORT);
@ISA = qw(Exporter);
@EXPORT = qw(
	am_get_statecodes 
	get_tcp_socket 
	get_udp_socket 
	get_sockaddr 
	my_system
);

# Read all the AM statecodes into a hash
# return the hash
sub am_get_statecodes
{
	# get the swatch statecodes
	open(STATES, "/usr/sausalito/swatch/statecodes") 
		|| die "can't open statecodes";

	my %am_states;
	while (defined($_ = <STATES>)) {
		if (/(AM_STATE_[A-Z]*)\s*=\s*(.*)/) {
			$am_states{$1} = $2;
		}
	}
	close(STATES);

	return %am_states;
}

# Return a handle to a socket
# Arguments:  host, port
sub get_tcp_socket
{
	my $host = shift || "localhost";
	my $port = shift || 80;

	my $sock;

	if ($port =~ /\D/) {
		$port = getservbyname($port, 'tcp');
	}

	$sock = new IO::Socket::INET->new(PeerAddr => $host,
		PeerPort => $port, Proto => 'tcp');

	return $sock;
}

# Fork and exec a process, closing file handles along the way
# Argument: the command to exec
sub my_system
{
	my $cmd = shift || "true";
	my $pid; 

	if ($pid = fork) {
		# parent
		wait;
		return $?
	} else {
		# child
		close(STDIN);	
		close(STDOUT);	

		open(STDIN, "</dev/null") || die "open: $!";
		open(STDOUT, ">/dev/null") || die "open: $!";

		exec($cmd);
		exit(42);
	}
}


# Return a handle to a socket
# Arguments: host, port
sub get_udp_socket
{
	my $host = shift || 'localhost';
	my $port = shift || 67;

	my $type = getprotobyname('udp');

	socket(SOCK, PF_INET, SOCK_DGRAM, $type);

	return *SOCK;
}

# Return a sockaddr_in
# Args: host, port
sub get_sockaddr
{
	my $host = shift || 'localhost';
	my $port = shift || 67;

	my $address = gethostbyname($host);
	my $sockaddr = sockaddr_in($port, $address);

	return $sockaddr;
}

1;
