#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: am_temp.pl,v 1.10 2002/02/20 04:19:43 uzi Exp $
# Copyright 2000, 2001 Sun Microsystems, Inc., All rights reserved.

use AM::Util;
use strict;
use CCE;

my $proc_thermal = "/proc/cobalt/sensors/thermal";
my $adm1029ctl = "/sbin/adm1029ctl";
my %am_states = am_get_statecodes();
my $red = 0;
my $yellow = 0;
my $sensor = {
    CPU  => { max => 80 },
    Case => { max => 58 },
};

if (-x $adm1029ctl and open ADM, "$adm1029ctl --dump |") {
    while (<ADM>) {
	if (/^cpu temp max: (-?[\d\.]+)$/) {
	    $sensor->{CPU}{max} = $1 if ($1 > 0);
	}
	elsif (/^board temp max: (-?[\d\.]+)$/) {
	    $sensor->{Case}{max} = $1 if ($1 > 0);
	}
    }
    close ADM;
}

for (keys %$sensor) {
    my $s = $sensor->{$_};
    $s->{AM_STATE_RED}    = 0.90 * $s->{max};
    $s->{AM_STATE_YELLOW} = 0.80 * $s->{max};
}

unless (-e $proc_thermal and open PROC, $proc_thermal) {
    print "[[base-am.cant_get_tempinfo]]";
    exit $am_states{AM_STATE_NOINFO};
}

while (<PROC>) {
    next unless /^\d+\s*\[(.*?)\]:\s*(-?[\d\.]+)$/;
    my $name = $1;
    my $temp = $2;
    next unless exists($sensor->{$name});
    my $s = $sensor->{$name};

    if ($temp > $s->{AM_STATE_RED}) {
	$red = 1;
    }
    elsif ($temp > $s->{AM_STATE_YELLOW}) {
	$yellow = 1;
    }
}
close PROC;
    
if ($red) {
    # sysfault light
    my $cce = new CCE();
    $cce->connectuds();
    my ($am_oid) = $cce->find('ActiveMonitor');
    my ($ok) = $cce->set($am_oid, 'Temp', { 'sysfault' => 1 });
    $cce->bye('SUCCESS');

    print "[[base-am.amTempsDangerouslyHot]]";
    exit $am_states{AM_STATE_RED};
}

if ($yellow) {
    print "[[base-am.amTempsHot]]";
    exit $am_states{AM_STATE_YELLOW};
}

print "[[base-am.amTempsOK]]";
exit $am_states{AM_STATE_GREEN};
